package rition.backend.api.v1.panel;

import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import rition.backend.api.v1.dto.request.MetricDataRequest;
import rition.backend.api.v1.dto.response.MetricDataResponse;
import rition.backend.api.v1.dto.response.Response;
import rition.common.data.entity.MetricRecordEntity;
import rition.service.panel.MetricService;

import java.time.Instant;
import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/panel/metrics")
public class MetricsViewController {

    private final MetricService metricService;

    public MetricsViewController(MetricService metricService) {
        this.metricService = metricService;
    }

    @GetMapping("/list")
    public Response<List<MetricDataResponse>> getMetrics(@RequestBody MetricDataRequest metricDataRequest) {
        var entityResult = metricService.getMetricDataRange(
                metricDataRequest.getInstanceId(),
                metricDataRequest.getMetricItems(),
                Instant.ofEpochMilli(metricDataRequest.getStart()),
                Instant.ofEpochMilli(metricDataRequest.getEnd())
        );

        // 转换数据
        List<MetricDataResponse> responseList = new ArrayList<>(entityResult.size());
        for (MetricRecordEntity entity : entityResult) {
            responseList.add(MetricDataResponse.builder()
                    .metricData(entity.getMetricData())
                    .time(entity.getTime())
                    .build()
            );
        }

        return Response.success(responseList);
    }
}
