package rition.service.panel;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import rition.common.data.dao.mapper.MetricRecordMapper;
import rition.common.data.entity.MetricRecordEntity;

import java.time.Instant;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

@Slf4j
@Service
public class MetricService {

    private final MetricRecordMapper metricRecordMapper;

    public MetricService(MetricRecordMapper metricRecordMapper) {
        this.metricRecordMapper = metricRecordMapper;
    }

    public List<MetricRecordEntity> getMetricDataRange(String instanceId,
                                                       List<String> metricItems,
                                                       Instant start, Instant end) {
        var filteredMetricItems = this.filterMetricItems(metricItems);
        if (filteredMetricItems.isEmpty()) {
            return new ArrayList<>();
        }

        return metricRecordMapper.getMetricDataGroupByHour(instanceId, metricItems, start, end);
    }

    private static final HashSet<String> allowedMetricItems = new HashSet<>(22);

    static {
        allowedMetricItems.add("up");
        allowedMetricItems.add("node_load5");
        allowedMetricItems.add("node_sockstat_TCP_tw");
        allowedMetricItems.add("node_cpu_seconds_total");
        allowedMetricItems.add("node_memory_Cached_bytes");
        allowedMetricItems.add("node_memory_Buffers_bytes");
        allowedMetricItems.add("node_memory_MemFree_bytes");
        allowedMetricItems.add("node_disk_read_bytes_total");
        allowedMetricItems.add("node_filesystem_free_bytes");
        allowedMetricItems.add("node_filesystem_size_bytes");
        allowedMetricItems.add("node_memory_MemTotal_bytes");
        allowedMetricItems.add("node_netstat_Tcp_CurrEstab");
        allowedMetricItems.add("node_filesystem_avail_bytes");
        allowedMetricItems.add("node_disk_written_bytes_total");
        allowedMetricItems.add("node_disk_reads_completed_total");
        allowedMetricItems.add("node_network_receive_drop_total");
        allowedMetricItems.add("node_disk_writes_completed_total");
        allowedMetricItems.add("node_network_receive_bytes_total");
        allowedMetricItems.add("node_network_transmit_drop_total");
        allowedMetricItems.add("node_network_transmit_bytes_total");
        allowedMetricItems.add("node_network_receive_packets_total");
        allowedMetricItems.add("node_network_transmit_packets_total");
    }

    public List<String> filterMetricItems(List<String> metricItems) {
        List<String> filteredMetricItems = new ArrayList<>(metricItems.size());
        for (String metricItem : metricItems) {
            if (allowedMetricItems.contains(metricItem)) {
                filteredMetricItems.add(metricItem);
            } else {
                log.warn("[MetricService]: 含有非允许的指标值：{}", metricItem);
            }
        }

        return filteredMetricItems;
    }
}
